//Windows header
#ifdef WIN32
#include <math.h>
#include <windows.h>
#ifndef M_PI
#  define M_PI 3.14159265358979323846f
# endif
#endif
// Glut for display window managment
#include <gl/glut.h>

#include <stdlib.h>
#include <time.h>
#include <iostream>

using namespace std;




/////////////////////////////////////////
//  Camera methods
////////////////////////////////////////

float cameraAngle=M_PI*0.5f, distToOrigin = 10.0, cameraHeight = 3.0f;


void setCamera() {
    float x = distToOrigin*cos(cameraAngle);
    float y = cameraHeight;
    float z = distToOrigin*sin(cameraAngle);

    glLoadIdentity();
    gluLookAt(x, y, z, -x, y, -z, 0.0f,1.0f,0.0f);
}

void changeSize(int w, int h)   {
    // Prevent a divide by zero, when window is too short
    // (you cant make a window of zero width).
    if(h == 0) h = 1;

    float ratio = 1.0f * w / h;
    // Reset the coordinate system before modifying
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    // Set the viewport to be the entire window
    glViewport(0, 0, w, h);

    // Set the clipping volume
    gluPerspective(45,ratio,1,1000);
    glMatrixMode(GL_MODELVIEW);
    setCamera();
}


////////////////////////////////////////
// Initialization methods
////////////////////////////////////////

void initScene() {
	// This Will Clear The Background Color To Black :
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

	// Clears The Depth Buffer :
	glClearDepth(1.0);
	// The Type Of Depth Test To Do :
	glDepthFunc(GL_LESS);
	// Enables Depth Testing :
	glEnable(GL_DEPTH_TEST);

	// Enables Smooth Color Shading :
	glShadeModel(GL_SMOOTH);
	// Controls the rendering style (fill, line or point) :
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glLineWidth(1.0f);
	glPointSize(4.5f);

	// init lighting :
	glEnable(GL_LIGHTING);
	glEnable(GL_COLOR_MATERIAL);
	GLfloat lightAmbient[]= { 0.5f, 0.5f, 0.5f, 1.0f };
	GLfloat lightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };
	GLfloat lightPosition[]= { 4.0*cos(60.0), 3.0f, 4.0*sin(60.0), 1.0f };
	glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
	glLightfv(GL_LIGHT0, GL_POSITION,lightPosition);
	glEnable(GL_LIGHT0);
}


//////////////////////////////////////////////
//  Draw methods
//////////////////////////////////////////////

void drawPlane(){
	float backColor[] = {1.0, 0.0, 0.0};
	float frontColor[] = {0.0, 0.0, 1.0};

	glBegin(GL_QUADS);
		glColor3fv(frontColor);
		glVertex3f(-10.0f, 0.0f, -10.0f);
		
		glColor3fv(backColor);
		glVertex3f(-10.0f, 0.0f,  10.0f);

		glColor3fv(backColor);
		glVertex3f( 10.0f, 0.0f,  10.0f);

		glColor3fv(frontColor);
		glVertex3f( 10.0f, 0.0f, -10.0f);
	glEnd();
}


void drawCube(){
	// TO DO
	
}



void drawCone(){
	int nbFaces = 50;
	float stepAngle = 2*M_PI/nbFaces;

	// draw base :
	// TO DO

	// draw apex :
	// TO DO

}

void drawCylinder(){
	int nbStrips = 20;
	int nbFacesPerStrip = 50;
	float stepHAngle = 2*M_PI/nbFacesPerStrip;

	// draw top :
	// TO DO

	// draw base :
	// TO DO

	// draw cylinder
	// TO DO
}

void drawSphere(){
    int nbStrips = 20;
    float stepVAngle = M_PI/nbStrips;
    int nbFacesPerStrip = 50;
    float stepHAngle = 2*M_PI/nbFacesPerStrip;

    for(int i=0 ; i<nbStrips ; i++) {
        float vAngle1 = i*stepVAngle - 0.5*M_PI;
        float vAngle2 = (i+1)*stepVAngle - 0.5*M_PI;

        // draw one strip :
        float intensity1 = (float)i/nbStrips;
        float intensity2 = (float)(i+1)/nbStrips;
        glBegin(GL_QUAD_STRIP);
        for (int j=0 ; j<=nbFacesPerStrip ; j++){
            float hAngle = j*stepHAngle;
            float pos1[] = {cos(hAngle)*cos(vAngle1), sin(vAngle1), sin(hAngle)*cos(vAngle1)};
            float pos2[] = {cos(hAngle)*cos(vAngle2), sin(vAngle2), sin(hAngle)*cos(vAngle2)};

            glColor3f(intensity1, 0.0, 1.0-intensity1);
            glNormal3fv(pos1);
            glVertex3fv(pos1);
            glColor3f(intensity2, 0.0, 1.0-intensity2);
            glNormal3fv(pos2);
            glVertex3fv(pos2);
        }
        glEnd();
    }

}

void renderScene(void) {
	// clear buffers
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	// draw objects
	drawPlane();

	glPushMatrix();
	glTranslatef(-4.0f ,1.0f, 0.0f);
	drawCube();

    glTranslatef(2.25f ,0.0f, 0.0f);
    drawCone();
    
	glTranslatef(2.25f ,0.0f, 0.0f);
    drawSphere();
    
	glTranslatef(2.25f ,0.0f, 0.0f);
    drawCylinder();

    glPopMatrix();
				
    // display rendering
	glutSwapBuffers();
}


///////////////////////////////////////
//  Callback methods
///////////////////////////////////////


void specialKey(int key, int x_, int y_) {

    switch (key) {
		case GLUT_KEY_LEFT :
			cameraAngle += 0.1f;
			setCamera();  break;
		case GLUT_KEY_RIGHT :
			cameraAngle -=0.1f;
			setCamera();  break;
		case GLUT_KEY_UP :
			distToOrigin -= 0.2f;
			if (distToOrigin<1.5f) distToOrigin = 1.5f;
			setCamera();  break;
		case GLUT_KEY_DOWN :
			distToOrigin += 0.2f;
			setCamera();  break;
		case GLUT_KEY_PAGE_UP :
			cameraHeight += 0.1f;
			setCamera();  break;
		case GLUT_KEY_PAGE_DOWN :
			cameraHeight -= 0.1f;
			setCamera();  break;
    }
}




void commonKey(unsigned char key, int x_, int y_) {
}






////////////////////////////////////
//  Main method
////////////////////////////////////

int main(int argc, char **argv) {
    // create window :
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
    glutInitWindowPosition(100,100);
    glutInitWindowSize(800, 600);
    glutCreateWindow("OpenGL");

    // initialize OpenGL states
    initScene();

    // Bind Key events
    glutSpecialFunc(specialKey);
    glutKeyboardFunc(commonKey);

    // Bind rendering methods
    glutDisplayFunc(renderScene);
    glutIdleFunc(renderScene);

    glutReshapeFunc(changeSize);

    // run program
    glutMainLoop();

    return(0);
}
